export interface Node {
    type: string;
    text: string;
    [key: string]: unknown;
}
export declare type Match = string | RegExp | Match.Location | Match.Location[] | Match.Function;
export declare namespace Match {
    type Location = [number, number];
    namespace Location {
        function is(loc: any): loc is Location;
    }
    type Function = (str: string) => string | RegExp | Location | Location[];
    function is(val: any): val is Match;
}
export declare type Replace = string | Replace.Function;
export declare namespace Replace {
    type Function = (match: string, ...groups: string[]) => Node | string;
    function is(val: any): val is Replace;
}
export interface Rule {
    match: Match;
    replace?: Replace;
}
export declare class Parser {
    static presets: Map<string, Match>;
    /**
     * Register a new global preset rule. Presets don't handle the replacing, only the matching. There are three
     * pre-included presets: `tag`, `url`, and `email`.
     */
    static registerPreset(name: string, match: Match): void;
    rules: Rule[];
    /** Add a rule to this parser. A rule consists of a match and optionally a replace. */
    addRule(match: Match, replace?: Replace): this;
    /**
     * Add a registered global preset rule within this parser and give it a replace. The preset must first be registered
     * using `Parser.registerPreset()` before it can be used with this method.
     */
    addPreset(name: string, replace?: Replace): this;
    private _replace;
    private _toTree;
    /**
     * Returns the parsed string as an array of nodes. Every node includes at least `type` and `text` properties. `type`
     * defaults to `"text"` but could be any value as returned by `replace`. The `text` key is used to replaced the
     * matched string by `render()`.
     */
    toTree(str: string): Node[];
    /**
     * Returns a parsed string with all matches replaced.
     */
    render(str: string): string;
}
export default Parser;
